/* Include the controller definition */
#include "epuck_aggregation.h"
/* Function definitions for XML parsing */
#include <argos3/core/utility/configuration/argos_configuration.h>

/****************************************/
/****************************************/

CEPuckAggregation::CEPuckAggregation() :
   m_pcWheels(NULL),
   m_pcProximity(NULL),
   m_WheelVelocityL0(2.5f),
   m_WheelVelocityR0(2.5f),
   m_WheelVelocityL1(2.5f),
   m_WheelVelocityR1(2.5f) {}

/****************************************/
/****************************************/

void CEPuckAggregation::Init(TConfigurationNode& t_node) {
   /*
    * Get sensor/actuator handles
    *
    * The passed string (ex. "differential_steering") corresponds to the
    * XML tag of the device whose handle we want to have. For a list of
    * allowed values, type at the command prompt:
    *
    * $ argos3 -q actuators
    *
    * to have a list of all the possible actuators, or
    *
    * $ argos3 -q sensors
    *
    * to have a list of all the possible sensors.
    *
    * NOTE: ARGoS creates and initializes actuators and sensors
    * internally, on the basis of the lists provided the configuration
    * file at the <controllers><epuck_obstacleavoidance><actuators> and
    * <controllers><epuck_obstacleavoidance><sensors> sections. If you forgot to
    * list a device in the XML and then you request it here, an error
    * occurs.
    */
   m_pcWheels    = GetActuator<CCI_EPuckWheelsActuator>("epuck_wheels");
   m_pcProximity = GetSensor  <CCI_EPuckProximitySensor>("epuck_proximity");
   /*
    * Parse the configuration file
    *
    * The user defines this part. Here, the algorithm accepts three
    * parameters and it's nice to put them in the config file so we don't
    * have to recompile if we want to try other settings.
    */
   GetNodeAttributeOrDefault(t_node, "velocity_l0", m_WheelVelocityL0, m_WheelVelocityL0);
   GetNodeAttributeOrDefault(t_node, "velocity_r0", m_WheelVelocityR0, m_WheelVelocityR0);
   GetNodeAttributeOrDefault(t_node, "velocity_l1", m_WheelVelocityL1, m_WheelVelocityL1);
   GetNodeAttributeOrDefault(t_node, "velocity_r1", m_WheelVelocityR1, m_WheelVelocityR1);
}

/****************************************/
/****************************************/

void CEPuckAggregation::ControlStep() {
   /* Get the highest reading in front of the robot, which corresponds to the closest object */
   Real fMaxReadVal = m_pcProximity->GetReading(0).Value;
   UInt32 unMaxReadIdx = 0;
   if(fMaxReadVal < m_pcProximity->GetReading(7).Value) {
      fMaxReadVal = m_pcProximity->GetReading(7).Value;
      unMaxReadIdx = 7;
   }

   /* Do we have an obstacle in front? */
   if(fMaxReadVal > 0.1f) {
     /* Yes, we do: avoid it */
     m_pcWheels->SetLinearVelocity(m_WheelVelocityL1, m_WheelVelocityR1);
   }
   else {
     /* No, we don't: go straight */
      m_pcWheels->SetLinearVelocity(m_WheelVelocityL0, m_WheelVelocityR0);
   }
}

/****************************************/
/****************************************/

/*
 * This statement notifies ARGoS of the existence of the controller.
 * It binds the class passed as first argument to the string passed as
 * second argument.
 * The string is then usable in the configuration file to refer to this
 * controller.
 * When ARGoS reads that string in the configuration file, it knows which
 * controller class to instantiate.
 * See also the configuration files for an example of how this is used.
 */
REGISTER_CONTROLLER(CEPuckAggregation, "epuck_aggregation_controller")
